<?php

namespace Drupal\requirements\Plugin;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Logger\LoggerChannelTrait;
use Drupal\Core\Messenger\MessengerTrait;
use Drupal\Core\Plugin\PluginBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines a base class for requirements plugins.
 */
abstract class RequirementsBase extends PluginBase implements RequirementsInterface {

  use RequirementsTrait;
  use LoggerChannelTrait;
  use MessengerTrait;

  /**
   * The requirements group this belongs to.
   *
   * @var \Drupal\requirements\Plugin\RequirementsGroupInterface
   */
  protected $requirement_group;

  /**
   * {@inheritdoc}
   */
  public function getId(): string {
    return $this->pluginDefinition['id'];
  }

  /**
   * {@inheritdoc}
   */
  public function getGroup():? RequirementsGroupInterface {
    if (empty($this->requirement_group) && !empty($this->pluginDefinition['group'])) {
      $groups = $this->getRequirementsGroupManager()->listRequirementsGroups();
      $group_id = $this->pluginDefinition['group'];
      $this->requirement_group = isset($groups[$group_id]) ? $groups[$group_id] : NULL;
    }

    return $this->requirement_group;
  }

  /**
   * {@inheritdoc}
   */
  public function getLabel(): String {
    return $this->pluginDefinition['label'];
  }

  /**
   * {@inheritdoc}
   */
  public function getDescription(): String {
    return $this->pluginDefinition['description'];
  }

  /**
   * {@inheritdoc}
   */
  public function getForm(): ?String {
    return $this->pluginDefinition['form'] ?? RequirementsFormBase::class;
  }

  /**
   * {@inheritdoc}
   */
  public function getActionButtonLabel(): ?String {
    return $this->pluginDefinition['action_button_label'] ?? NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function getSeverity(): String {
    return $this->isCompleted() ? 'completed' : ($this->pluginDefinition['severity'] ?? 'warning');
  }

  /**
   * {@inheritdoc}
   */
  public function getDependencies(): array {
    return $this->pluginDefinition['dependencies'] ?? [];
  }

  /**
   * {@inheritdoc}
   */
  public function getActionButton(): array {
    if (!$this->getActionButtonLabel()) {
      return [];
    }

    return Link::createFromRoute($this->getActionButtonLabel(), "requirements.{$this->getId()}", [
      \Drupal::destination()->getAsArray(),
    ], [
      'attributes' => [
        'data-dialog-type' => 'modal',
        'data-dialog-options' => json_encode([
          'width' => 600,
          'height' => 450,
          'draggable' => FALSE,
          'autoResize' => FALSE,
        ]),
        'class' => [
          'use-ajax',
          'button',
        ],
      ],
    ])->toRenderable();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state): array {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {

  }

  /**
   * {@inheritdoc}
   */
  public function isResolvable(): bool {
    // TODO: Implement a dependency tree so we can avoid circular dependencies.
    foreach ($this->getDependencies() as $dependency) {
      if (!$this->getRequirementsManager()->createInstance($dependency)->isCompleted()) {
        return FALSE;
      }
    }

    return TRUE;
  }

}
